package com.wrj.driver.server.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.dto.QuestionPracticeAddDto;
import com.wrj.driver.server.dto.QuestionPracticeQueryDto;
import com.wrj.driver.server.entity.WrjQuestion;
import com.wrj.driver.server.entity.WrjQuestionPractice;
import com.wrj.driver.server.entity.WrjQuestionPracticeTotal;
import com.wrj.driver.server.entity.WrjUserErrorQuestion;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.mapper.WrjQuestionMapper;
import com.wrj.driver.server.mapper.WrjQuestionPracticeMapper;
import com.wrj.driver.server.mapper.WrjQuestionPracticeTotalMapper;
import com.wrj.driver.server.mapper.WrjUserErrorQuestionMapper;
import com.wrj.driver.server.service.IWrjQuestionPracticeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.wrj.driver.server.vo.QuestionPracticeVo;
import com.wrj.driver.server.vo.QuestionVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 用户答题练习表 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2025-06-29
 */
@Service
public class WrjQuestionPracticeServiceImpl extends ServiceImpl<WrjQuestionPracticeMapper, WrjQuestionPractice> implements IWrjQuestionPracticeService {

    @Autowired
    private WrjQuestionMapper questionMapper;

    @Autowired
    private WrjQuestionPracticeMapper practiceMapper;

    @Autowired
    private WrjUserErrorQuestionMapper errorQuestionMapper;

    @Autowired
    private WrjQuestionPracticeTotalMapper practiceTotalMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean insertQuestionPractice(QuestionPracticeAddDto addDto) {

        WrjQuestion question = questionMapper.selectById(addDto.getQuestionId());
        if (question == null) {
            throw new BusinessException("题目不存在");
        }
        boolean isTrue = StrUtil.equals(addDto.getAnswer(), question.getTrueAnswer());

        WrjQuestionPractice questionPractice = new WrjQuestionPractice();
        BeanUtil.copyProperties(addDto, questionPractice);

        // 插入练习
        questionPractice.setPracticeTime(LocalDateTime.now());
        questionPractice.setAnswerResult(isTrue? Constants.STR_ONE : Constants.STR_TWO);
        practiceMapper.insert(questionPractice);

        // 判断答题是否正确

        questionMapper.updateAnswerNumber(addDto.getQuestionId(), 1, (isTrue ? 1 : 0));

        if (!isTrue) {
            // 判断是否在错题本里（答题错误会加入错题本，答题正确不会移出）
            WrjUserErrorQuestion userErrorQuestion = errorQuestionMapper.selectByUserIdAndQuestionId(addDto.getUserId(), addDto.getQuestionId());
            if (userErrorQuestion == null) {
                userErrorQuestion = new WrjUserErrorQuestion()
                        .setQuestionId(addDto.getQuestionId())
                        .setUserId(addDto.getUserId())
                        .setCreateTime(LocalDateTime.now());
                // 将错误的答题加入错题本中
                errorQuestionMapper.insert(userErrorQuestion);
            }
        }
        // 个人练习的统计
        WrjQuestionPracticeTotal questionPracticeTotal = practiceTotalMapper.selectByUserIdAndQuestionId(addDto.getUserId(), addDto.getQuestionId());
        if (questionPracticeTotal == null) {
            questionPracticeTotal = new WrjQuestionPracticeTotal();
            questionPracticeTotal.setQuestionId(addDto.getQuestionId());
            questionPracticeTotal.setUserId(addDto.getUserId());
            questionPracticeTotal.setAnswerNumber(1l);
            questionPracticeTotal.setRightNumber((isTrue ? 1l : 0l));
            practiceTotalMapper.insert(questionPracticeTotal);
        } else {
            practiceTotalMapper.updateAnswerNumber(addDto.getUserId(), addDto.getQuestionId(), 1, (isTrue ? 1 : 0));
        }

        return Boolean.TRUE;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean clear(Long userId, String practiceType) {
        practiceMapper.clearPracticeRecord(userId, practiceType);

        return Boolean.TRUE;
    }

    @Override
    public IPage<QuestionPracticeVo> query(QuestionPracticeQueryDto queryDto) {

        Page<QuestionPracticeVo> page = new Page<>(queryDto.getPageNo(), queryDto.getPageSize());

        IPage<QuestionPracticeVo> pageResult = practiceMapper.queryQuestionPracticePage(page, queryDto);

        questionOptionHandler(pageResult.getRecords());

        return pageResult;
    }

    /**
     * 问题选项规整
     */
    private void questionOptionHandler(List<QuestionPracticeVo> questionVos) {
        if (CollectionUtil.isNotEmpty(questionVos)) {
            for (QuestionPracticeVo questionVo : questionVos) {
                //如果选项A没有值 则为判断题
                List<QuestionVo.QuestionOption> optionList = new ArrayList<>();
                if (StrUtil.isBlank(questionVo.getChooseA()) || StrUtil.equals(questionVo.getType(), Constants.QUESTION_TYPE_ONE)) {
                    optionList.add(new QuestionVo.QuestionOption("A", "正确", "A"));
                    optionList.add(new QuestionVo.QuestionOption("B", "错误", "B"));
                } else {
                    //起码ABCD都有
                    optionList.add(new QuestionVo.QuestionOption("A", questionVo.getChooseA(), "A"));
                    optionList.add(new QuestionVo.QuestionOption("B", questionVo.getChooseB(), "B"));
                    optionList.add(new QuestionVo.QuestionOption("C", questionVo.getChooseC(), "C"));
                    optionList.add(new QuestionVo.QuestionOption("D", questionVo.getChooseD(), "D"));

                    if (StrUtil.isNotBlank(questionVo.getChooseE())) {
                        optionList.add(new QuestionVo.QuestionOption("E", questionVo.getChooseE(), "E"));
                    }
                    if (StrUtil.isNotBlank(questionVo.getChooseF())) {
                        optionList.add(new QuestionVo.QuestionOption("F", questionVo.getChooseF(), "F"));
                    }
                    if (StrUtil.isNotBlank(questionVo.getChooseG())) {
                        optionList.add(new QuestionVo.QuestionOption("G", questionVo.getChooseG(), "G"));
                    }
                }
                questionVo.setOptionList(optionList);
            }
        }
    }
}
